package lottery;

import java.io.FileInputStream;
import java.rmi.*;
import java.util.*;

import zyneo.sms.*;
import zyneo.sms.gateway.adaptor.rmi.client.*;

/**
* Copyright (c) 2002 Zyneo
*
* $Id: Server.java,v 1.1 2003/03/26 09:10:01 nian Exp $
*
* @author nian
* 
* <pre>
* $Log: Server.java,v $
* Revision 1.1  2003/03/26 09:10:01  nian
* Added file
*
* 
* </pre>
*/

/**
*
* This our powerful lottery server :-)
* 
* A lottery consists of a set of tickets
* were one ticket is the winning ticket.
* 
* Participants send a SMS with their name
* to the lottery server and there by "purchases"
* a ticket. Each user are only allowed to by one
* ticket, this is enforced by recording the
* users mobile numbers.
* 
* When all tickets are sold we draw a winner and
* sends a SMS notifying the winner that he/she has
* won. We also sends a SMS message to each of the
* other participants notifying them who won.
* 
* The server can be configured by using a properties
* file which is accepted as the first cmd-line argument.
* 
* Have fun !
* 
* The Server uses the Java-RMI adaptor for transmission
* and reception of messages. The reason for this is
* no other than that we want to show an example with
* the Java-RMI adaptor. We could just as easy have used 
* the XML/tcp Adaptors, and to show how they would be used 
* there are alternate send(...) and receive (..) metods 
* using the XML/tcp adaptors in in asociation to the "real"
* methods used.
*
*/
public class Server
{
	/** Our properties */
	
	/** The rmi-url for the SmsRMI-Adaptor */
	public static final String RMI_URL = "rmi-url";
	public static final String RMI_URL_DEFAULT = "rmi://localhost/rmi-adaptor";
	protected String m_rmi_url;
	
	/** The number of tickets */
	public static final String NUM_TICKETS = "tickets";
	public static final int NUM_TICKETS_DEFAULT = 5;
	protected int m_tickets;
	
	/** Message sent as a response to a user which participates in the lottery */
	public static final String PARTICIPATE_MESSAGE = "participate-message";
	public static final String PARTICIPATE_MESSAGE_DEFAULT = "Thank you! You participates in the lottery.";
	protected String m_participate_message;
		
	/** Message sent to user when he/she tries to participate several times in the same lottery */
	public static final String DUPLICATE_MESSAGE = "duplicate-message";
	public static final String DUPLICATE_MESSAGE_DEFAULT = "Sorry, you are only allowed to participate once in this lottery.";
	protected String m_duplicate_message;
	
	/** Message sent to the user which won the lottery */
	public static final String WIN_MESSAGE = "win-message";
	public static final String WIN_MESSAGE_DEFAULT = "Congratulations!!! You have won the lottery and the price an Ultra Toaster Deluxe!.";
	protected String m_win_message;
	
	/** Message sent to all users which did not win, the name of the user who won will be appended to the end of this message */
	public static final String LOOSE_MESSAGE = "loose-message";
	public static final String LOOSE_MESSAGE_DEFAULT = "Sorry, you didn't win.The winner was ";
	protected String m_loose_message;
	
	/** The factory object used for creating RMI-Sockets */
	protected Factory m_factory;
	
	/** All our participants */
	protected Participant[] m_participants;
	protected int m_sold_tickets;
	
	/** A set containing all registered mobile phone numbers,
	 * we use this to keep track of cheaters !
	 */ 
	protected TreeSet m_numbers;
	
	
	/**
	 * Constructor for Lottery.
	 */
	public Server()
	{
		super();
		
		// Initialize default values
		m_rmi_url = RMI_URL_DEFAULT;
		m_tickets = NUM_TICKETS_DEFAULT;
		m_participate_message = PARTICIPATE_MESSAGE_DEFAULT;
		m_duplicate_message = DUPLICATE_MESSAGE_DEFAULT;
		m_win_message = WIN_MESSAGE_DEFAULT;
		m_loose_message = LOOSE_MESSAGE_DEFAULT;
		
		m_factory = null;
	}
	
	/**
	*
	* Configures this server using supplied properties
	*
	* @param props The properties to use
	*
	* @throws IllegalArgumentException
	*/
	public void configure(Properties props)
		throws IllegalArgumentException
	{
		// Fetch all properties
		m_rmi_url = props.getProperty(RMI_URL);
		if(m_rmi_url == null || "".equals(m_rmi_url))
			throw new IllegalArgumentException("The property " + RMI_URL + " must not be empty");
		
		m_tickets = Integer.parseInt(props.getProperty(NUM_TICKETS));

		if(m_tickets < 1)
			throw new IllegalArgumentException("You must at least have one ticket");

		// We allow this messages to be empty, and if they are
		// we simply not send them.
		m_participate_message = props.getProperty(PARTICIPATE_MESSAGE);
		m_duplicate_message = props.getProperty(DUPLICATE_MESSAGE);			
		m_win_message = props.getProperty(WIN_MESSAGE);
		m_loose_message = props.getProperty(LOOSE_MESSAGE);
	}
	
	
	/**
	*
	* Our main loop
	* 
	*/
	public void run()
	{
		try
		{
			log("Starts a new Lottery ...");
			
			// Lets connect to the Gateway
			log("Connecting to Gateway on:" + m_rmi_url);

			m_factory = (Factory)Naming.lookup(m_rmi_url);
			
			
			// And initialze our participant collection
			m_participants = new Participant[m_tickets];
			m_sold_tickets = 0;
			m_numbers = new TreeSet();
					
			// And lets start the lottery
			while(m_sold_tickets < m_tickets)
			{
				log("Availible tickets:" + (m_tickets - m_sold_tickets));
				
				Sms ticket = receive();
								
				// OK, we have one sold ticket lets
				// register this participant, but only 
				// if it's not registered before.
				if(!m_numbers.contains(ticket.getFrom()))
				{
					// No, this mobile phone number is not
					// registered before.
					m_numbers.add(ticket.getFrom());
					
					// And lets save info about the participant
					String name = ticket.getMessage();
					
					// If the user sent an empty SMS,
					// we use the mobile number as name.
					if(name == null || "".equals(name))
						name = ticket.getFrom();
					
					m_participants[m_sold_tickets] = new Participant(ticket.getFrom(),name);
					
					m_sold_tickets++;
					
					// And lets log ..
					log("Sold a ticket to:" + name + " with mobile number:" + ticket.getFrom());
					
					// And send a message to participant.
					send(m_participate_message,ticket.getFrom());	
				}
				else
				{
					// We have a cheater !!!!
					log("Received: " + ticket.getMessage() + " from: " + ticket.getFrom() + ", this message will be ignored since the user already has bought a ticket");
					send(m_duplicate_message,ticket.getFrom());
				}
			}
			
			log("All tickets are sold, drawing a winner ...");
			
			// Lets sleep a while to generate some exitement!
			Thread.sleep(5000);
			
			// We have sold all tickets, lets draw a winner.
			Random r = new Random(System.currentTimeMillis());
			
			// And we pick the index of the winner
			int winner_index = r.nextInt(m_tickets + 1);
			Participant winner = m_participants[winner_index]; 
			log("And the winner is " + winner.getName() + "!!!!");
			
			// Lets send him the winner message
			send(m_win_message,winner.getMobilenumber());
			
			// And send all others the looser message.
			for(int i = 0; i < m_tickets; i++)
			{
				if(i != winner_index)
				{
					send(this.m_loose_message + winner.getName(),m_participants[i].getMobilenumber());
				}
			}
			
			// And the lottery is all done
			log("Lottery is finished...");
						
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	*
	* Receives a SMS-message, since we are only interested
	* in SMS-messages all status-reports (instances of Notification)
	* will be ignored.
	*
	*
	* @return A SMS-message received
	* 
	* @throws RemoteException, InterruptedException
	*/
	public Sms receive() 
		throws RemoteException, InterruptedException
	{
		ReceiveSocket rs = null;
		

		try
		{
			// And lets create the ReceiveSocket which we
			// will use for reception of SMS-messages
			rs = m_factory.createReceiveSocket();
			
			// And lets wait for someone to buy a ticket
			Sms ticket = null;

			while(ticket == null)
			{
				// Check how many objects that are
				// availible
				int num = rs.size();

				if(num > 0)
				{
					// Ahh, we got some object lets fetch them
					// and see what they are ...
					while(rs.size() > 0 && ticket == null)
					{
						Object o = rs.receivenb();

						// We are only interested in SMS-messages
						if(o instanceof zyneo.sms.Sms)
						{
							// We have a message
							ticket = (Sms)o;
						}
					}
				}
				else
				{
					// Lets sleep a while
					Thread.sleep(5000);
				}
			}
			
			return ticket;			
		}
		finally
		{
			if(rs != null)
				rs.close();
		}
	}

	/**
	*
	* Alternate receive-method which uses the
	* XML/tcp Receive Adaptor instead of Java-RMI.
	* 
	* @return A SMS-message received
	*
	* @throws UnknownHostException, RequestException, IOException
	*/
/*	public Sms receive() 
		throws UnknownHostException, RequestException, IOException
	{
		// Lets create a connection, if we were
		// to use this implementation we would
		// off course read hostname and port from
		// a properties file, but here they are
		// hardcoded.
		String hostname = "localhost";
		int port = 3001;
		
		ReceiveConnection rc = new ReceiveConnection(hostname,port);

		// Connect connection to server
	
		// And lets wait for someone to buy a ticket
		Sms ticket = null;

		while(ticket == null)
		{
			// Check how many objects that are
			// availible
			Object o = rc.receive();

			// We are only interested in SMS-messages
			if(o instanceof zyneo.sms.Sms)
			{
				// We have a message
				ticket = (Sms)o;
			}
		}

		return ticket;
	}
*/	
	/**
	*
	* Sends a message. If the message is empty, the message
	* will silently be ignored.
	*
	* @param m The message to send
	* @param to The mobile number to send to
	*
	* @throws RemoteException, InterruptedException
	*/
	public void send(String m,String to) 
		throws RemoteException, InterruptedException
	{
		SendSocket ss = null;
		
		try
		{
			
			if(m != null && !"".equals(m))
			{
				ss = m_factory.createSendSocket();
				
				// Lets send the message
				Sms sms = new Sms(to,m);
				
				log("Sending: " + m + " to: " + to);
				
				// Since we keep no track of the sent
				// messages we will ignore the message-identifier
				ss.send(sms);
			}
			else
			{
				log("The message is empty, no message will be sent to: " + to);
			}
		}
		finally
		{
			ss.close();
		}
	}
	
	/**
	*
	* Send-method using the XML/tcp Send Adaptor instead
	* of using the Java-RMI adaptor.
	* 
	* This method is equivalent to the send-method
	* above.
	*
	* @param m The message to send
	* @param to The mobile number to send to
	*
	* @return A Sms message received.
	* @throws UnknownHostException, RequestException, IOException
	*/
/*	public void send(String m,String to) 
		throws UnknownHostException, RequestException, IOException
	{
		SendConnection conn = null;

		try
		{

			if(m != null && !"".equals(m))
			{
				// The connection properties of the
				// XML/tcp adaptor would off course be
				// read from a properties file if we were
				// to use this send(..) implementation.
				// 
				// Here they are hardcoded.
				//
				String hostname = "localhost";
				int port = 3000;
								
				conn = new SendConnection(hostname,port);

				// Lets send the message
				Sms sms = new Sms(to,m);

				log("Connecting to gateway on " + hostname + " port " + port);
				
				conn.connect();
				
				log("Sending: " + m + " to: " + to);

				// Since we keep no track of the sent
				// messages we will ignore the message-identifier
				conn.send(sms);
			}
			else
			{
				log("The message is empty, no message will be sent to: " + to);
			}
		}
		finally
		{
			if(conn != null)
				conn.disconnect();
		}
	}
*/	
	
	
	/**
	*
	* Main method, simply starts a new lottery.
	* 
	* It accepts the following cmd-line:
	* 
	* Server [properties-file]
	*
	* @param args The commandline arguments
	*
	*/
	public static void main(String[] args)
	{
		try
		{
			Server server = new Server();
			
			// Lets see if we are to load a config-file
			if(args.length > 0)
			{
				// Lets load the properties
				Properties props = new Properties();
				props.load(new FileInputStream(args[0]));
				
				// And configure the server
				server.configure(props);
			}
			
			// And boost up the server
			server.run();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	*
	* Simple function to log to Std-out
	*
	* @param m The log message
	*
	*/
	public void log(String m)
	{
		System.out.println(m);
	}
}
