from base.Logger import *

import socket
import time

class SocketConnection:

    """
        $Id: SocketConnection.py,v 1.1.1.1 2003/04/10 14:38:57 maro Exp $
        
        Provides a simple socket connection class used to send messages to
        the gateway, connects using blocking sockets.

        @author maro
    """

    STATE_RUNNING = "running"
    STATE_STOPPED = "stopped"

    m_host = "localhost"
    m_port = 3001
    m_listener = None
    m_socket = None
    m_run = None
    m_state = None
    m_log = None

    def __init__(self, host, port):
        """Constructor, creates a new socket."""
        self.m_log = Logger()
        self.m_host = host
        self.m_port = port
        self.m_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    def addMessageListener(self,listener):
        """Adds a listener which should be called when messages arrive from
            the gateway"""
        self.m_listener = listener

    def listen(self):
        """Connects to the gateway using blocking sockets and waits
            for messages. When messages arrive, the listener is
            notified."""
        self.m_log.logInfo("Connecting to tcp//%s:%d" % (self.m_host, self.m_port))
        try:
            self.m_socket.connect((self.m_host, self.m_port))
            self.m_log.logInfo("Connected, waiting for messages...")
            self.m_run = 1
            self.m_state = self.STATE_RUNNING
            while self.m_run:
                chunk = self.m_socket.recv(4096)
                if chunk != "":
                    self.m_listener.onMessageReceive(chunk)
        except socket.error, msg:
            self.m_log.logFatal("Could not connect! %s" % msg)
        except socket.herror, msg:
            self.m_log.logFatal("Could not connect! %s" % msg)
        except socket.gaierror, msg:
            self.m_log.logFatal("Could not connect! %s" % msg)

        self.m_socket.close()

    def shutdown(self):
        print "   Shutting down socket-connection."
        self.m_run = None
        self.m_socket.close()
        self.m_state = self.STATE_STOPPED

    def getState(self):
        return self.m_state

    def send(self,msg):
        """Sends a message through the socket opened by listen method
            does not close the socket after sending
            Used by the GatewayListener to send responses on received messages"""
        self.m_socket.send(msg)

    def sendReceive(self, msg):
        """Sends the message to host and port defined in the constructor and
            returns the response from the socket after closing it.
            Used by the GatewaySender"""
        self.m_log.logInfo("Connecting to tcp//%s:%d" % (self.m_host, self.m_port))
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect((self.m_host, self.m_port))
            self.m_log.logDebug("Connected")
            self.m_log.logDebug("Sending message: %s " % msg)
            unimsg = unicode(msg, "latin-1")
            s.send(unimsg.encode("utf-8"))
            self.m_log.logDebug("Message sent")
            while 1:
                chunk = s.recv(4096)
                if ( chunk != "" ):
                    s.close()
                    return chunk
        except socket.error, msg:
            self.m_log.logFatal("Could not connect! %s" % msg)
            return None
        except socket.herror, msg:
            self.m_log.logFatal("Could not connect! %s" % msg)
            return None
        except socket.gaierror, msg:
            self.m_log.logFatal("Could not connect! %s" % msg)
            return None

